@@ -398,7 +398,8 @@ c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
       do i=1,nvar
          do j=1,nvar
-            propv(i,j)=sigma(i,j)
+c            propv(i,j)=sigma(i,j)
+            propv(i,j)=0.d0
          end do
       end do
       
@@ -731,7 +732,7 @@ c+++++++ Candidate generating kernel
          
          do i=1,nvar
             do j=1,nvar
-               sigmac(i,j)=(nu-dble(nvar-1))*sigma(i,j)
+               sigmac(i,j)=(nu-dble(nvar+1))*sigma(i,j)
             end do
          end do
 
@@ -757,11 +758,11 @@ c+++++++ Candidate generating kernel
          logcgkn=logcgkn+nu*detlog2
 
          tmp2=0.d0
-         do i=1,2
-            do j=1,2
+         do i=1,nvar
+            do j=1,nvar
                tmp1=0.d0 
-               do k=1,2
-                  tmp1=tmp1+(nu-dble(nvar-1))*
+               do k=1,nvar
+                  tmp1=tmp1+(nu-dble(nvar+1))*
      &                       sigmac(i,k)*sigmainv(k,j)        
                end do
                if(i.eq.j)tmp2=tmp2+tmp1
@@ -775,11 +776,11 @@ c+++++++ Candidate generating kernel
          logcgko=logcgko+nu*detlog1
          
          tmp2=0.d0
-         do i=1,2
-            do j=1,2
+         do i=1,nvar
+            do j=1,nvar
                tmp1=0.d0 
-               do k=1,2
-                  tmp1=tmp1+(nu-dble(nvar-1))*
+               do k=1,nvar
+                  tmp1=tmp1+(nu-dble(nvar+1))*
      &                       sigma(i,k)*sigmainvc(k,j)        
                end do
                if(i.eq.j)tmp2=tmp2+tmp1
@@ -802,10 +803,11 @@ c+++++++ End candidate generating kernel
 
          do i=1,nvar
             do j=1,nvar
-               propv(i,j)=sigmac(i,j)
+c               propv(i,j)=sigmac(i,j)
+                propv(i,j)=0.d0 
             end do
          end do
-         call cholesky(nvar,propv,workmh)
+         call cholesky(nvar,sigmac,workmh)
       
          do i=1,nvar
             do j=1,i
@@ -1002,9 +1004,9 @@ c++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 c++++++++++ sample candidates
 
-            cparc=rtlnorm(log(cpar),tune3*0.2,0,0,.true.,.true.)
-            logcgkn=dlnrm(cpar ,log(cparc),tune3*0.2,1) 
-            logcgko=dlnrm(cparc,log(cpar ),tune3*0.2,1) 
+            cparc=rtlnorm(log(cpar),tune3*1.0,0,0,.true.,.true.)
+            logcgkn=dlnrm(cpar ,log(cparc),tune3*1.0,1) 
+            logcgko=dlnrm(cparc,log(cpar ),tune3*1.0,1) 
 
 c++++++++++ evaluate log-prior for candidate value of the parameters
 
@@ -1220,7 +1222,7 @@ c++++++++++++++++ check if the user has requested an interrupt
             
                   loglikn=loglikn+
      &              log((2.d0**nvar)*cpar+dble((2.d0**nvar)*countero))-
-     &              log((2.d0**nvar)*cpar+dble(i-1))
+     &              log((2.d0**nvar)*cpar+dble(nrec-1))
 
                   if(countero.eq.0) go to 5
 
@@ -1653,51 +1655,3 @@ c+++++++++++++ print
       end
       
       
-c=======================================================================                  
-      integer function binaryrep(nvar,pattern)
-c=======================================================================                  
-c     function that return the integer number+1, based on its
-c     binary representation
-c     Alejandro Jara, 2006
-
-      implicit none
-      integer i,nvar
-      integer pattern(nvar)
-      
-      binaryrep=0
-      do i=1,nvar
-         binaryrep=binaryrep+(2**(i-1))*pattern(i)
-      end do
-      binaryrep=binaryrep+1
-      return
-      end
-
-
-c=======================================================================                  
-      subroutine binaryrepinv(nvar,evali,pattern)
-c=======================================================================                  
-c     function that return the binary representation of number 
-c     given evali=number+1
-c     Alejandro Jara, 2006
-
-      implicit none
-      integer i,nvar,evali,tmp1
-      integer pattern(nvar)
-      real*8 tmp2
-      
-      evali=evali-1
-      do i=1,nvar
-         tmp1=int(evali/2.d0)
-         tmp2=dble(evali)/2.d0
-         if(abs(tmp2-tmp1).gt.0.d0)then
-           pattern(i)=1
-          else
-           pattern(i)=0
-         end if  
-         evali=tmp1
-      end do
-      return
-      end
-
-
-
